## Code for Diamond Cruise data as an example
## set sensitivity=0.70 specificity=0.95

## code for Figure 1: the Diamond Cruise example

## Assuming the current directory is the working directory

## Read data 
dat<-read.csv("Dia_Princess.csv", head=T)

## col 2: population, col 3: positives per day col 4: death on the day
## The data come from European CDC, may not be the same as the data from other places

ActualPass<-3711-cumsum(dat[,3])  ## if positive case were removed from the ship
## The total number of people on the cruise is 3711

R<-cumsum(dat[,3])/3711  ## assuming cases were not removed

R1<-cumsum(dat[,3])/ActualPass  ## assumed cases were removed

## P(Y=1|Y^*=1) 

RprobK<-function(x,y,z=0.001, K=1) {
 1/(1+(y/(1-x))^(K-1)*(1-y)/x*(1-z)/z) }

## P(Y=0|Y^*=0) 

RNprob<-function(x,y,z=0.001) {
 1/(1+(1-x)*z/y/(1-z)) }


PnumFpos<-3711 *(1-RprobK(0.70, 0.95, R, K=1))

APnumFpos<-ActualPass *(1-RprobK(0.70, 0.95, R1, K=1))

PnumFneg<-3711 *(1-RNprob(0.70, 0.95, R))

APnumFneg<-ActualPass *(1-RNprob(0.70, 0.95, R1))


par(mfrow=c(1,3))

plot(dat[,5], R1, type="l", lty=1,
     xlab="Days from Jan 19", ylab="Prevalence Rate",
     main="Diamond Cruise COVID-19 Data")


plot(dat[,5], APnumFpos, type="l", lty=2, xlab="Days from Jan 19", ylab="Numbers",
        main="All Passengers Tested", col="blue",ylim=c(0, 3711))

lines(dat[,5], ActualPass,lty=1, col="red")

legend(0, 2500, c("Passengers on board", "False Positves"), lty=1:2, col=c("red","blue"))




plot(dat[,5], APnumFneg, type="l",lty=2,xlab="Days from Jan 19", ylab="Numbers",
        main="All Passengers Tested", col="blue")

##lines(dat[,5], ActualPass,lty=1, col="red")
 

#legend(0, 2000, c("Passengers on board", "False Negatives"), lty=1:2, col=c("red","blue"))


## file name "DiamondCruise_New.pdf"


#################################################################################
## Code for figure 2:
library(plot3D)



Psen<-c(0.4, 0.45, 0.5, 0.55, 0.6, 0.65,0.7, 0.75, 0.8, 0.85, 0.9)
Pspe<-c(0.4, 0.45, 0.5, 0.55, 0.6, 0.65,0.7, 0.75, 0.8, 0.85, 0.9)

R<-c(0.01, 0.25, 0.85)

RprobK<-function(x,y,z=0.001, K=1) {
 1/(1+(y/(1-x))^(K-1)*(1-y)/x*(1-z)/z) }


par(mfrow=c(1,3))


PProb<-outer(Psen,Pspe, z=R[1], K=3, RprobK)

persp3D(Psen, Pspe, PProb, xlab="Sensitivity", ylab="Specificity", clim=c(0,1), 
##theta=30, phi=50, axes=TRUE,scale=2, box=TRUE, nticks=5,   ticktype="detailed",
colkey = FALSE,
       main=paste("K=3, Prevalence=", R[1]))

PProb<-outer(Psen,Pspe, z=R[2], K=3, RprobK)

persp3D(Psen, Pspe, PProb, xlab="Sensitivity", ylab="Specificity", clim=c(0,1), 
##theta=30, phi=50, axes=TRUE,scale=2, box=TRUE, nticks=5,   ticktype="detailed",
colkey = FALSE,
       main=paste("K=3, Prevalence=", R[2]))


PProb<-outer(Psen,Pspe, z=R[3], K=3, RprobK)

persp3D(Psen, Pspe, PProb, xlab="Sensitivity", ylab="Specificity", clim=c(0,1), 
##theta=30, phi=50, axes=TRUE,scale=2, box=TRUE, nticks=5,   ticktype="detailed",
colkey = list(side = 4, length = 0.8),
       main=paste("K=3, Prevalence=", R[3]))



#################################################################################
## Code for figure 3


par(mfrow=c(1,2))

## P(Y=1|Y^*_1=0, ..., Y^*_K-1=0, Y^*_K=1)

RprobK<-function(x,y,z=0.001, K=1) {
 1/(1+(y/(1-x))^(K-1)*(1-y)/x*(1-z)/z) }


R<-c(0.00001, 0.0005, 0.001, 0.005, 0.01, 0.05, 0.1, 0.15, 0.20, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5, 0.55, 
        0.6, 0.65,0.7, 0.75, 0.8, 0.85, 0.9,0.95)

plot(R, RprobK(0.8866, 0.9063, z=R, K=2), type="l", col=1, xlab="Prevalence", ylab="Chance of  Being Infected",
       ylim=c(0.0, 1.0),
       main="Sensitivity = 0.8866, Specificity=0.9063")
lines(R, RprobK(0.8866, 0.9063, z=R, K=3), lty=2, col=2)
lines(R, RprobK(0.8866, 0.9063, z=R, K=4), lty=3, col=3)
lines(R, RprobK(0.8866, 0.9063, z=R, K=5), lty=4, col=4)
lines(R, RprobK(0.8866, 0.9063, z=R, K=6), lty=5, col=5)


legend(0.0, 1.0, paste("K= ", 2:6), col=1:5, lty=1:5)




R<-c(0.00001, 0.0005, 0.001, 0.005, 0.01, 0.05, 0.1, 0.15, 0.20, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5, 0.55, 
        0.6, 0.65,0.7, 0.75, 0.8, 0.85, 0.9,0.95)

plot(R, RprobK(0.70, 0.95, z=R, K=2), type="l", col=1, xlab="Prevalence", ylab="Chance of Being Infected",
       ylim=c(0.0, 1.0),
       main="Sensitivity = 0.70, Specificity=0.95")
lines(R, RprobK(0.70, 0.95, z=R, K=3), lty=2, col=2)
lines(R, RprobK(0.70, 0.95, z=R, K=4), lty=3, col=3)
lines(R, RprobK(0.70, 0.95, z=R, K=5), lty=4, col=4)
lines(R, RprobK(0.70, 0.95, z=R, K=6), lty=5, col=5)



legend(0.0, 1.0, paste("K= ", 2:6), col=1:5, lty=1:5)

## file "PNagNTestsPos.pdf"



#################################################################################
## Code for figure 4




## P(Y=1|Y^*_1=0, ..., Y^*_K=0)



Psen<-c(0.4, 0.45, 0.5, 0.55, 0.6, 0.65,0.7, 0.75, 0.8, 0.85, 0.9)
Pspe<-c(0.4, 0.45, 0.5, 0.55, 0.6, 0.65,0.7, 0.75, 0.8, 0.85, 0.9)

R<-c(0.01, 0.25, 0.85)

RprobK<-function(x,y,z=0.001, K=1) {
 1/(1+(y/(1-x))^K*(1-z)/z) }

par(mfrow=c(1,3))



PProb<-outer(Psen,Pspe, z=R[1], K=3, RprobK)

persp3D(Psen, Pspe, PProb, xlab="Sensitivity", ylab="Specificity", clim=c(0,1), 
#theta=30, phi=50, axes=TRUE,scale=2, box=TRUE, nticks=5,   ticktype="detailed",
colkey = FALSE,
       main=paste("K=3, Prevalence=", R[1]))


PProb<-outer(Psen,Pspe, z=R[2], K=3, RprobK)

persp3D(Psen, Pspe, PProb, xlab="Sensitivity", ylab="Specificity", clim=c(0,1), 
#theta=30, phi=50, axes=TRUE,scale=2, box=TRUE, nticks=5,   ticktype="detailed",
colkey = FALSE,
       main=paste("K=3, Prevalence=", R[2]))

PProb<-outer(Psen,Pspe, z=R[3], K=3, RprobK)

persp3D(Psen, Pspe, PProb, xlab="Sensitivity", ylab="Specificity", clim=c(0,1), 
#theta=30, phi=50, axes=TRUE,scale=2, box=TRUE, nticks=5,   ticktype="detailed",
colkey = list(side = 4, length = 0.8),
       main=paste("K=3, Prevalence=", R[3]))



## file name: NConsecTest_new.pdf


##################################################################################
## Code for Figue 5



par(mfrow=c(1,2))

##P(Y=1|Y^*_1=0, ..., Y^*_K=0)

RprobK<-function(x,y,z=0.001, K=1) {
 1/(1+(y/(1-x))^K*(1-z)/z) }


R<-c(0.00001, 0.0005, 0.001, 0.005, 0.01, 0.05, 0.1, 0.15, 0.20, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5, 0.55, 
        0.6, 0.65,0.7, 0.75, 0.8, 0.85, 0.9,0.95)

plot(R, RprobK(0.8866, 0.9063, z=R, K=2), type="l", col=1, xlab="Prevalence", ylab="Prob of 
       Missing an Infected Case", ylim=c(0.0, 0.7),
       main="Sensitivity = 0.8866, Specificity=0.9063")
lines(R, RprobK(0.8866, 0.9063, z=R, K=3), lty=2, col=2)
lines(R, RprobK(0.8866, 0.9063, z=R, K=4), lty=3, col=3)
lines(R, RprobK(0.8866, 0.9063, z=R, K=5), lty=4, col=4)
lines(R, RprobK(0.8866, 0.9063, z=R, K=6), lty=5, col=5)


legend(0.0, 0.7, paste("K= ", 2:6), col=1:5, lty=1:5)




R<-c(0.00001, 0.0005, 0.001, 0.005, 0.01, 0.05, 0.1, 0.15, 0.20, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5, 0.55, 
        0.6, 0.65,0.7, 0.75, 0.8, 0.85, 0.9,0.95)

plot(R, RprobK(0.70, 0.95, z=R, K=2), type="l", col=1, xlab="Prevalence", ylab="Prob of 
       Missing an Infected Case",ylim=c(0.0, 0.7),
       main="Sensitivity = 0.70, Specificity=0.95")
lines(R, RprobK(0.70, 0.95, z=R, K=3), lty=2, col=2)
lines(R, RprobK(0.70, 0.95, z=R, K=4), lty=3, col=3)
lines(R, RprobK(0.70, 0.95, z=R, K=5), lty=4, col=4)
lines(R, RprobK(0.70, 0.95, z=R, K=6), lty=5, col=5)



legend(0.0, 0.7, paste("K= ", 2:6), col=1:5, lty=1:5)


## file: "PNagNTests.pdf"


##########################################################################
